#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#define	inpfpath		"BUILD.INP"
#define	outpfpath		"BUILD.OUT"

#define	W_MAX			11
#define	H_MAX			11

void sys_err (const char *format, ...)
{
 	va_list			va;

        va_start (va, format);
        fprintf (stderr, "** ERROR **:\n\t");
        vfprintf (stderr, format, va);
        fprintf (stderr, ".\n\tLast system error: %i %s.\n\n",
                 errno, strerror (errno));
        va_end (va);
}

typedef	struct Pt_ts		Pt;
typedef	struct Tile_ts		Tile;

struct Pt_ts {
	int			X,
				Y;
};

struct Tile_ts {
	Pt			A,
				B;
};

unsigned int			w, h;
int                             canv [W_MAX][H_MAX];
unsigned int			sol;

#define	TILES_C			2

Tile                            tiles [TILES_C] =
{
	/* Horiz: */ { {0, 0}, {1, 0} },
	/* Vert: */  { {0, 0}, {0, 1} }
};

int tl_can_put (unsigned int x, unsigned int y, unsigned int tl_id)
{
	int			x2, y2;

	x2 = x + tiles [tl_id].B.X;
	y2 = y + tiles [tl_id].B.Y;
	if ( x2 >= w || y2 >= h ) {
		return (0);
	}
	if ( canv [x][y] || canv [x2][y2] ) {
		return (0);
	}
	return (1);
}

unsigned int		tl_left;

void tl_put (unsigned int x, unsigned int y, unsigned int tl_id)
{
	int		x2, y2;

	x2 = x + tiles [tl_id].B.X;
	y2 = y + tiles [tl_id].B.Y;
	canv [x][y] = 1;
	canv [x2][y2] = 1;
	tl_left--;
}

void tl_unput (unsigned int x, unsigned int y, unsigned int tl_id)
{
	int		x2, y2;

	x2 = x + tiles [tl_id].B.X;
	y2 = y + tiles [tl_id].B.Y;
	canv [x][y] = 0;
	canv [x2][y2] = 0;
	tl_left++;
}

void try_all (unsigned int x0, unsigned int y0)
{
	unsigned int		x, y, tl_id;

	if ( tl_left == 0 ) {
		sol++;
		return;
	}
	x = x0;
	for ( y = y0; y < h; y++ ) {
		for ( ; x < w; x++ ) {
			for ( tl_id = 0; tl_id < TILES_C; tl_id++ ) {
				if ( !tl_can_put (x, y, tl_id) ) {
					continue;
				}
				tl_put (x, y, tl_id);
				try_all (x, y);
				tl_unput (x, y, tl_id);
			}
		}
		x = 0;
	}
}

int go (void)
{
	unsigned int		x, y;

	sol = 0;
	if ( (w*h) % 2 == 1 ) {
		return (0);
	}
	for ( y = 0; y < h; y++ ) {
		for ( x = 0; x < w; x++ ) {
			canv [x][y] = 0;
		}
	}
	tl_left = (w*h)/2;
	try_all (0, 0);
	return (0);
}

int init (void)
{
	FILE			*inpf;

	errno = 0;
        if ( (inpf = fopen (inpfpath, "r")) == NULL ) {
        	sys_err ("Unabe to open the input file ('%s')", inpfpath);
                return (-1);
	}
	if ( fscanf (inpf, "%u %u\n", &w, &h) != 2 ) {
		sys_err ("Error inputting W and H");
		fclose (inpf);
		return (-1);
	}
	if ( fclose (inpf) ) {
        	sys_err ("Unable to close the output file ('%s')",
                         outpfpath);
                return (-1);
        }
	return (0);
}

int shut (void)
{
	FILE		*outpf;

        if ( (outpf = fopen (outpfpath, "w")) == NULL ) {
        	sys_err ("Unable to open the output file ('%s')", outpfpath);
                return (-1);
	}
	fprintf (outpf, "%u\n", sol);
	if ( ferror (outpf) ) {
          	sys_err ("Error writing to the output file ('%s')",
                         outpfpath);
               	fclose (outpf);
                return (-1);
        }
	if ( fclose (outpf) ) {
        	sys_err ("Unable to close the output file ('%s')", outpfpath);
                return (-1);
        }
	return (0);
}

int main (void)
{
 	if ( init () ) {
        	return (1);
        }
	if ( go () ) {
        	return (1);
 	}
	if ( shut () ) {
        	return (1);
	}
        return (0);
}
