#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#define	inpfpath		"ROUTES.INP"
#define	outpfpath		"ROUTES.OUT"

#define	TOWN_C_MAX		170
#define	CPL_C_MAX		( (TOWN_C_MAX * (TOWN_C_MAX-1)) / 2 )

void sys_err (const char *format, ...)
{
 	va_list			va;

        va_start (va, format);
        fprintf (stderr, "** ERROR **:\n\t");
        vfprintf (stderr, format, va);
        fprintf (stderr, ".\n\tLast system error: %i %s.\n\n",
                 errno, strerror (errno));
        va_end (va);
}

typedef	struct Cpl_ts		Cpl;

struct Cpl_ts {
	unsigned int		A,
        			B;
};

int     	*links [TOWN_C_MAX];
char		*sol [CPL_C_MAX];
unsigned int    n, cpl_c;

int are_linked (unsigned int a, unsigned int b)
{
	int		alinks [TOWN_C_MAX][TOWN_C_MAX], flag;
        unsigned int    c, d;

	for ( d = 0; d < n; d++ ) {
		for ( c = 0; c < n; c++ ) {
			alinks [d][c] = links [d][c];
		}
	}
	flag = 1;
        while ( flag ) {
          	flag = 0;
		c = 0;
                while ( !alinks [a][c] ) {
                  	if ( c >= n ) {
                                return (0);
                        }
                	c++;
                }
                for ( d = 0; d < n; d++ ) {
			if ( alinks [c][d] ) {
				if ( d == b ) {
					return (1);
				}
				alinks [a][d] = 1;
                                alinks [d][a] = 1;
                        }
                }
        }
	return (0);
}

int strcmpfn (const void *e1, const void *e2)
{
        return (strcmp ( * (char**) e1, * (char**) e2 ));
}

int go (void)
{
	unsigned int		a, b;

        for ( a = 0; a < n; a++ ) {
        	for ( b = a+1; b < n; b++ ) {
                        if ( !links [a][b] ) {
                        	continue;
                        }
                	links [a][b] = 0;
                        links [b][a] = 0;
			if ( !are_linked (a, b) ) {
				sprintf (sol [cpl_c], "%u %u", a+1, b+1);
                                cpl_c++;
                        }
                	links [a][b] = 1;
                        links [b][a] = 1;
                }
        }
	qsort (&sol [0], cpl_c, sizeof (sol [0]), strcmpfn);
	return (0);
}

int init (void)
{
	FILE			*inpf;
	unsigned int		a, b, c, i, cpl_c;

	errno = 0;
	cpl_c = 0;
	for ( c = 0; c < TOWN_C_MAX; c++ ) {
		links [c] = (int*) malloc (sizeof (int)*TOWN_C_MAX);
		if ( links [c] == NULL ) {
			sys_err ("No memory for graph");
			return (-1);
		}
	}
	for ( a = 0; a < TOWN_C_MAX; a++ ) {
	   for ( b = 0; b < TOWN_C_MAX; b++ ) {
	      links [a][b] = 0;
	   }
	}
	if ( (inpf = fopen (inpfpath, "r")) == NULL ) {
		sys_err ("Unabe to open the input file ('%s')", inpfpath);
		return (-1);
	}
	if ( fscanf (inpf, "%u\n", &n) != 1 ) {
		sys_err ("Error inputting N");
		fclose (inpf);
		return (-1);
	}
	cpl_c = (n * (n-1))/2;
	for ( c = 0; c < cpl_c; c++ ) {
		sol [c] = (char*) malloc (sizeof (char)*10);
		if ( sol [c] == NULL ) {
			sys_err ("No memory for soulutions array");
			return (-1);
		}
	}
	for ( c = 0; c < n; c++ ) {
		if ( fscanf (inpf, "%u ", &i) != 1 ) {
			sys_err ("Error inputting route number %u", c+1);
			fclose (inpf);
			return (-1);
		}
		for ( ; i != 0; i-- ) {
			if ( fscanf (inpf, "%u", &b) != 1 ) {
				sys_err ("Error inputting route number %u",
                                         c+1);
                        	fclose (inpf);
                        	return (-1);
                        }
 			links [c][b-1] = 1;
                        links [b-1][c] = 1;
                }
        }
	if ( fclose (inpf) ) {
        	sys_err ("Unable to close the output file ('%s')",
                         outpfpath);
                return (-1);
        }
	return (0);
}

int shut (void)
{
	FILE		*outpf;
        unsigned int	c;

        if ( (outpf = fopen (outpfpath, "w")) == NULL ) {
        	sys_err ("Unable to open the output file ('%s')", outpfpath);
                return (-1);
        }
	for ( c = 0; c < cpl_c; c++ ) {
                fprintf (outpf, "%s\n", sol [c]);
        }
	if ( ferror (outpf) ) {
          	sys_err ("Error writing to the output file ('%s')",
                         outpfpath);
               	fclose (outpf);
                return (-1);
        }
	if ( fclose (outpf) ) {
        	sys_err ("Unable to close the output file ('%s')", outpfpath);
                return (-1);
        }
	return (0);
}

int main (void)
{
 	if ( init () ) {
        	return (1);
        }
	if ( go () ) {
        	return (1);
 	}
	if ( shut () ) {
        	return (1);
	}
        return (0);
}
